// CRR3Engine.cpp - give us a nice API for sending device commands here a device command is a single keystroke on the remote.
// See CVCRAction for higher level commands.
/*
This software is provided 'as-is', without any express or implied   warranty.  In no event will the authors be held liable for any damages   arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,   including commercial applications, and to alter it and redistribute it   freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not      claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be      appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be      misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.
	David Phillip Oster <mailto:oster@ieee.org> Dec 15, 2003
 */
#include "stdafx.h"
#include "CRR3Engine.h"
#include "resource.h"
#include "StBSTR.h"
#include <memory>	// home of auto_ptr

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


namespace {

void MyDefaultReporter(LPCTSTR inMesg, void * /*environ*/){
	AfxMessageBox(inMesg, MB_ICONSTOP);
}


} // end anonymous namespace


CRR3Engine::CRR3Engine(LPCTSTR dbPath) : m_dbPath(dbPath){
	m_rrLookup = NULL;
	m_rr3 = NULL;
	m_signalDB = NULL;
	m_ReporterFunc = &MyDefaultReporter;
	m_ReporterEnviron = NULL;
	// OK, now it is safe to call Die() if we need to,
	try{
		Init();	// if an error occurs during construction...
	}catch(...){
		Die();	// ... clean up
		throw;
	}

}

void CRR3Engine::Init(){
 // Create a COM RedRatLookup object
	HRESULT hr;

	hr = CoCreateInstance(CLSID_RedRatLookup,
		NULL,
		CLSCTX_SERVER,
		IID_IRedRatLookup,
		reinterpret_cast<void**>(&m_rrLookup));
	if(FAILED(hr)){
		CString s;
		s.Format(IDS_NOCOMRR3);
		(*m_ReporterFunc)(s, m_ReporterEnviron);
		AfxThrowUserException();
	}

    StBSTR rrName(CString(_T("RedRat3-0")).AllocSysString());
    hr = m_rrLookup->GetRedRat3(rrName, &m_rr3);
	if(FAILED(hr)){
		CString s;
		s.Format(IDS_NORR3);
		(*m_ReporterFunc)(s, m_ReporterEnviron);
		AfxThrowUserException();
	}

	hr = CoCreateInstance(CLSID_CCWSignalDB,
        NULL,
		CLSCTX_SERVER,
        IID_ISignalDB,
	  	reinterpret_cast<void**>(&m_signalDB));
	if(FAILED(hr)){
		AfxThrowOleException(hr);
	}

	StBSTR dbFileName(m_dbPath.AllocSysString());
	hr = m_signalDB->AddDeviceDB(dbFileName);
	if(FAILED(hr)){
		CString s;
		s.Format(IDS_NORR3DB, static_cast<LPCTSTR>(m_dbPath));
		(*m_ReporterFunc)(s, m_ReporterEnviron);
		AfxThrowUserException();
	}
}

CRR3Engine::~CRR3Engine(){
	Die();
}

// note: this is the only member that can be called with a partially constructed object.
// all the other member functions can assume that all the data members are non-null.
void CRR3Engine::Die(){
	if(NULL != m_signalDB){ m_signalDB->Release(); }
	if(NULL != m_rr3){ m_rr3->Release(); }
	if(NULL != m_rrLookup){ m_rrLookup->Release(); }
}

void CRR3Engine::IREmit(LPCTSTR devNam, LPCTSTR keyNam){
	ASSERT(devNam);
	ASSERT(keyNam);
	CCWModulatedSignal sig;
    StBSTR devName(CString(devNam).AllocSysString());
    StBSTR sigName(CString(keyNam).AllocSysString());
    HRESULT hr = m_signalDB->GetSignal(devName, sigName, &sig);
	if(FAILED(hr)){
		CString s;
		s.Format(IDS_NOSIGNAL, keyNam, devNam, static_cast<LPCTSTR>(m_dbPath));
		(*m_ReporterFunc)(s, m_ReporterEnviron);
		AfxThrowUserException();
    }
	m_rr3->OutputModulatedSignal(&sig);
	if(FAILED(hr)){
		CString s;
		s.Format(IDS_COULDNTOUTPUT, keyNam, devNam);
		(*m_ReporterFunc)(s, m_ReporterEnviron);
		AfxThrowUserException();
	}
}

/// true if the device exists in the database
bool CRR3Engine::IRExists(LPCTSTR devNam){
	ASSERT(devNam);
    StBSTR devName(CString(devNam).AllocSysString());
	CCWAVDeviceInfo info;
	return SUCCEEDED(m_signalDB->GetAVDeviceInfo(devName, &info));
}

// true if command for this device exists in the database.
bool CRR3Engine::IRExists(LPCTSTR devNam, LPCTSTR keyNam){
	ASSERT(devNam);
	ASSERT(keyNam);
	CCWModulatedSignal  outSig;
    StBSTR devName(CString(devNam).AllocSysString());
    StBSTR sigName(CString(keyNam).AllocSysString());
	return SUCCEEDED(m_signalDB->GetSignal(devName, sigName, &outSig));
}


void CRR3Engine::Blink(){
	m_rr3->Blink();
}

// SetErrorReporter -
/// allow client code to pass in a different reporter, for example, to log messages to a file.
std::pair<CRR3Engine::ReporterFunc, void*> CRR3Engine::SetErrorReporter(ReporterFunc inFunc, void *environ){
	std::pair<CRR3Engine::ReporterFunc, void*>	oldValue(m_ReporterFunc, m_ReporterEnviron);
	m_ReporterFunc = (NULL != inFunc) ? inFunc : &MyDefaultReporter;
	m_ReporterEnviron = environ;
	return oldValue;
}

